# -*- coding: utf-8 -*-
## PloneSubscription
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module implement a folder subscription with recursivity ans workflow options
"""
__version__ = "$Revision: 1.12 $"
# $Source: /cvsroot/ingeniweb/PloneSubscription/content/FolderSubscription.py,v $
# $Id: FolderSubscription.py,v 1.12 2005/10/14 08:21:17 encolpe Exp $
__docformat__ = 'restructuredtext'


# Python imports
from AccessControl import ClassSecurityInfo
from Acquisition import aq_inner, aq_parent

# Zope imports
from DateTime import DateTime

# CMF imports
from Products.CMFCore  import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.public import *
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import *

# Products imports
from Products.PloneSubscription.config import PROJECTNAME
from Products.PloneSubscription.content.schemata import BaseSubscriptionSchema
from Products.PloneSubscription import SubscriptionPermissions
from Products.PloneSubscription.interfaces.ISubscription import ISubscription
from Products.PloneSubscription.SubscriptionCatalogAware import SubscriptionCatalogAwareContent



schema = BaseSubscriptionSchema.copy() + Schema ((
    ReferenceField(
        'folder',
        multiValued=False,
        required=True,
        searchable=False,
        relationship='subscriptionFolderOf',
        read_permission=SubscriptionPermissions.ViewSubscriptionContent,
        write_permission=SubscriptionPermissions.EditSubscriptionContent,
        widget=ReferenceBrowserWidget(
            addable=True,
            visible={ 'view': 'hidden', 'edit': 'visible',},
            label="Folder subscribed",
            label_msgid="label_subscription_folder",
            description="The folderish document on which the subscription is done.",
            description_msgid="help_subscription_folder",
            i18n_domain="plonesubscription",
            default_search_index='Title',
            available_indexes={'Title': 'title',}
            ),
        ),
    BooleanField(
        'recursive',
        multiValued=False,
        required=False,
        searchable=False,
        default=False,
        read_permission=SubscriptionPermissions.ViewSubscriptionContent,
        write_permission=SubscriptionPermissions.ViewSubscriptionContent,
        widget=BooleanWidget(
            addable=True,
            visible={ 'view': 'visible', 'edit': 'visible',},
            label="Recursive",
            label_msgid="label_subscription_recursive",
            description="Recursive search.",
            description_msgid="help_subscription_recursive",
            i18n_domain="plonesubscription",
            ),
        ),
    BooleanField(
        'workflow',
        multiValued=False,
        required=False,
        searchable=False,
        default=False,
        read_permission=SubscriptionPermissions.ViewSubscriptionContent,
        write_permission=SubscriptionPermissions.ViewSubscriptionContent,
        widget=BooleanWidget(
            addable=True,
            visible={ 'view': 'visible', 'edit': 'visible',},
            label="Workflow transition",
            label_msgid="label_subscription_workflow",
            description="Only return documents which passed a worlflow transition",
            description_msgid="help_subscription_workflow",
            i18n_domain="plonesubscription",
            ),
        ),
    ),)

class FolderSubscription(SubscriptionCatalogAwareContent):
    """Implement folder subscription"""

    __implements__ = BaseContent.__implements__, ISubscription

    portal_type = meta_type = 'FolderSubscription'
    archetype_name = 'FolderSubscription'
    immediate_view = 'base_view'
    default_view   = 'base_view'
    content_icon = 'foldersubscription_icon.gif'
    global_allow = False
    schema =  schema
    allowed_content_types = ()
    include_default_actions = 0
    security = ClassSecurityInfo()

    actions = (
        {
        'id'            : 'view',
        'name'          : 'View',
        'action'        : 'string:${object_url}/base_view',
        'permissions'   : (SubscriptionPermissions.ViewSubscriptionContent, ),
        'category'      : 'object',
        'visible'       : 1,
        },
        {
        'id'            : 'edit',
        'name'          : 'Edit',
        'action'        : 'string:${object_url}/base_edit',
        'permissions'   : (SubscriptionPermissions.ViewSubscriptionContent, ),
        'category'      : 'object',
        'visible'       : 1,
        },
    )

    # Fields related methods

    security.declareProtected(SubscriptionPermissions.ViewSubscriptionContent, 'getRpath')
    def getRpath(self, rpath=''):
        """Overload rpath accessor to synchronise reference"""
        folder = self.getFolder()
        if folder is None:
            return self.getField('rpath').get(self)
        else:
            return folder.absolute_url(relative=1)

    security.declareProtected(SubscriptionPermissions.EditSubscriptionContent, 'setRpath')
    def setRpath(self, rpath='/'):
        """Overload rpath mutator"""
        folder = self.getField('folder').get(self)
        if folder is None:
            self.getField('rpath').set(self, rpath)
        else:
            self.getField('rpath').set(self, folder.absolute_url(relative=1))

    # Interface related methods

    security.declareProtected(SubscriptionPermissions.ViewSubscriptionContent, 'getSubscriber')
    def getSubscriber(self):
        """Return the subscriber"""
        return aq_parent(aq_inner(self))

    security.declareProtected(SubscriptionPermissions.ViewSubscriptionContent, 'getQuery')
    def getQuery(self):
        """Return the request used for catalog query"""
        path_arg = None
        # customizing request for limited path search
        if not self.getRecursive():
            path_arg = {
                    'query': self.getRpath(),
                    'depth': 0,
                    }
        else:
            path_arg = self.getRpath()

        return {
            'path': path_arg,
            'ModificationDate': {
                'query': [self.getLast_send(),],
                'range': 'min'
                },
            }

    security.declareProtected(SubscriptionPermissions.ViewSubscriptionContent, 'getQueryBrains')
    def getQueryBrains(self):
        """Return the list of brains concerned by the mailing"""
        now = DateTime()
        last_send = self.getLast_send()
        periodicity = float(getToolByName(self, 'portal_subscription').getPeriodicity())
        if float(now - last_send) > periodicity/1440.0:
            ctool = getToolByName(self, 'portal_catalog')
            kwargs = self.getQuery()
            return ctool.searchResults(**kwargs)
        else:
            return []

    def afterQueryFilters(self, brains):
        """Filter query results with subscription options"""

        if hasattr(self, 'getWorkflow') and self.getWorkflow():
            results = []
            for brain in brains:
                history = wtool.getInfoFor(brain.getObject(), 'review_history', [])
                for transition in history:
                    if transition.has_key('time') and transition['time'] > last_send:
                        brains.append(brain)
                        break;

            return results
        else:
            return brains

    def getSubscriptionUrl(self):
        """Return the url of the object or concept subscribed on"""
        return self.getRpath()

registerType(FolderSubscription, PROJECTNAME)